/*
 * Network Explorer
 *
 * Copyright (C) 1998, Mark Spencer
 * 
 * Distributed under the terms of the GNU GPL
 *
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <arpa/inet.h>
#include <string.h>
#include <netdb.h>
#include "cheops.h"

#define FLAG_DOMAIN 1

static char *get_dom(char *s)
{
	char *s2=s;
	while(s2 && !get_server(s2, 0)) {
		s2 = strchr(s2, '.');
		if (s2)
			s2++;
	}
	if (!s2)
		s2=s;
	return s2;
}

static void place_object(struct net_object *no, int xp, int yp)
{
	struct link *l;
	gtk_widget_set_uposition(no->eventbox, xp, yp);
	gtk_widget_set_uposition(no->otherbox, xp, yp + ICON_HEIGHT - 20);
	l = no->links;
	while(l) {
		/* Check validity of other widget? */
		gtk_link_set_coords(GTK_LINK(l->linkw),
				    no->eventbox->allocation.x + no->eventbox->allocation.width/2,
				    no->eventbox->allocation.y + no->eventbox->allocation.height/2,
				    l->other->eventbox->allocation.x + l->other->eventbox->allocation.width / 2,
				    l->other->eventbox->allocation.y + l->other->eventbox->allocation.height / 2);
			l=l->next;
	}
}


static void place(struct net_object **nos, int cnt, struct net_page *np, int flags)
{
	int x;
	int xp, yp;
	char *last=NULL;
	struct in_addr ia;
	xp = 20;
	yp = 20;
	for (x=0;x<cnt;x++) {
		if (flags & FLAG_DOMAIN) {
			ia.s_addr = nos[x]->ip_addr;
			if (strcasecmp(nos[x]->hostname, inet_ntoa(ia))) {
				if (last) {
					if (strcasecmp(last, get_dom(nos[x]->hostname)) && (xp != 20)) {
						xp = 20;
						yp += ICON_HEIGHT;
						if (yp + ICON_HEIGHT > np->eventbox->allocation.height) {
							gtk_widget_set_usize(np->eventbox, 0, 
								np->eventbox->allocation.height + ICON_HEIGHT);
						}
					}
				}
				last = get_dom(nos[x]->hostname);
			} else last="";
		}
		place_object(nos[x], xp, yp);
		xp += ICON_WIDTH;
		if (xp > (option_use_visible_area ? np->pane->allocation.width : FIXED_WIDTH) - ICON_WIDTH) {
			xp = 20;
			yp += ICON_HEIGHT;
			if (yp + ICON_HEIGHT > np->eventbox->allocation.height) {
				gtk_widget_set_usize(np->eventbox, 0, np->eventbox->allocation.height + ICON_HEIGHT);
			}
		}
	}
	np->lastx = xp;
	np->lasty = yp;
}

static int compare_ip(const void *a_d, const void *b_d)
{
	struct net_object **a, **b;
	unsigned int aa, ab;
	a = (struct net_object **)a_d;
	b = (struct net_object **)b_d;
	aa=ntohl((*a)->ip_addr);
	ab=ntohl((*b)->ip_addr);
	if (aa<ab) return -1; else
	if (aa>ab) return 1; else
	return 0;
}

static int compare_os(const void *a_d, const void *b_d)
{
	struct net_object **a, **b;
	a = (struct net_object **)a_d;
	b = (struct net_object **)b_d;
	return strcasecmp((*a)->fn, (*b)->fn);
}

static int compare_name(const void *a_d, const void *b_d)
{
	struct net_object **a, **b;
	a = (struct net_object **)a_d;
	b = (struct net_object **)b_d;
	return strcasecmp((*a)->hostname, (*b)->hostname);
}

static int compare_domain(const void *a_d, const void *b_d)
{
	struct net_object **a, **b;
	int res;
	struct in_addr ia;
	char *da, *db;
	a = (struct net_object **)a_d;
	b = (struct net_object **)b_d;
	ia.s_addr = (*a)->ip_addr;
	if (!strcmp((*a)->hostname, inet_ntoa(ia))) 
		return strcasecmp((*a)->hostname, (*b)->hostname);
	ia.s_addr = (*b)->ip_addr;
	if (!strcmp((*b)->hostname, inet_ntoa(ia))) 
		return strcasecmp((*a)->hostname, (*b)->hostname);
	da = get_dom((*a)->hostname);
	db = get_dom((*b)->hostname);
	res = strcasecmp(da, db);
	if (!res)
		res = strcasecmp((*a)->hostname, (*b)->hostname);
	return res;
}


void arrange(struct net_page *np, int how)
{
	int cnt=0;
	int x=0;
	struct net_object *no;
	struct net_object **sorter;
	immediate_setcursor(GDK_WATCH);
	for (no = np->objs; no; no=no->next)
		cnt++;
	if (!cnt)
		return;
	sorter = g_new(struct net_object *, cnt);
	for (no = np->objs; no; no=no->next) {
		sorter[x]=no;
		x++;
	}
	switch(how) {
	case BY_OS:
		qsort(sorter, cnt, sizeof(struct net_object *), compare_os);
		place(sorter, cnt, np, 0);
		break;
	case BY_NAME:
		qsort(sorter, cnt, sizeof(struct net_object *), compare_name);
		place(sorter, cnt, np, 0);
		break;
	case BY_ADDR:
		qsort(sorter, cnt, sizeof(struct net_object *), compare_ip);
		place(sorter, cnt, np, 0);
		break;
	case BY_DOMAIN:
		qsort(sorter, cnt, sizeof(struct net_object *), compare_domain);
		place(sorter, cnt, np, FLAG_DOMAIN);
		break;
	default:
		fprintf(stderr, "Don't know how to arrange by %d\n", how);
	}
	g_free(sorter);
	gdk_window_set_cursor(main_window.window->window, NULL);
}
